/***************************************************************************
  This is a library for the BMP3XX temperature & pressure sensor

  Designed specifically to work with the Adafruit BMP388 Breakout
  ----> http://www.adafruit.com/products/3966

  These sensors use I2C or SPI to communicate, 2 or 4 pins are required
  to interface.

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing products
  from Adafruit!

  Written by Limor Fried & Kevin Townsend for Adafruit Industries.
  BSD license, all text above must be included in any redistribution
 ***************************************************************************/


/* Written By Walter Lohmueller
 *  This is a library for a simple altimeter that is intended for 
 *  use in a model rocket. 
 *  It uses a BMP 388 sensor to collect Temperature, Pressure and altitude
 *  data and logs said data to a micro SD card.
 *  This sketch is featured in my article on the NAR Sport Rocketry magazine
 *  Link Here:
 *  
 *  Have fun using this sketch and modify it any way you like!
 */

#include <Wire.h>//include the Wire.h for I2C communication
#include <SPI.h>//include SPI.h for SPI communication
#include <SD.h>// include SD.h for the micro SD card library.
#include <Adafruit_Sensor.h>
#include "Adafruit_BMP3XX.h"// include the BMP 388 library

#define BMP_SCK 13//define SPI pins for the BMP 388
#define BMP_MISO 12
#define BMP_MOSI 11
#define BMP_CS 10

#define SEALEVELPRESSURE_HPA (1013.25)

Adafruit_BMP3XX bmp;
float altitudeoffset = 0; //define the altitude offset
float actual_altitude = 0;//define the actual altitude

int chipSelect = 4; //chipSelect pin for the SD card Reader
File mySensorData; //Data object you will write your sesnor data to

void setup() {
  Serial.begin(115200);//start the serial monitor
  SD.begin(4); //Initialize the SD card reader
  Serial.println("Adafruit BMP388 / BMP390 test");

  //if (!bmp.begin_I2C()) {   // hardware I2C mode, can pass in address & alt Wire
  if (! bmp.begin_SPI(BMP_CS)) {  // hardware SPI mode  
  //if (! bmp.begin_SPI(BMP_CS, BMP_SCK, BMP_MISO, BMP_MOSI)) {  // software SPI mode
    Serial.println("Could not find a valid BMP3 sensor, check wiring!");
    while (1);
  }

  // Set up oversampling and filter initialization
  bmp.setTemperatureOversampling(BMP3_OVERSAMPLING_8X);
  bmp.setPressureOversampling(BMP3_OVERSAMPLING_4X);
  bmp.setIIRFilterCoeff(BMP3_IIR_FILTER_COEFF_3);
  bmp.setOutputDataRate(BMP3_ODR_50_HZ);
}

void loop() {
  
  if (! bmp.performReading()) {//check if the sensor has preformed a reading
    Serial.println("Failed to perform reading :(");
    return;
  }
  
  if(millis()/1000 < 5) { // if the time since power on is less than five seconds define the altitude offset as altitude above sea level
altitudeoffset = bmp.readAltitude(SEALEVELPRESSURE_HPA);
  }
    actual_altitude = (bmp.readAltitude(SEALEVELPRESSURE_HPA)) - altitudeoffset;//cacualte altitue above sea level
    
  Serial.print("Temperature = ");//print out all the data
  Serial.print(bmp.temperature);
  Serial.print(" *C");

  Serial.print("Pressure = ");
  Serial.print(bmp.pressure / 100.0);
  Serial.print(" hPa");

  Serial.print("Approx. Altitude = ");
  Serial.print(bmp.readAltitude(SEALEVELPRESSURE_HPA));
  Serial.print(" m");

  Serial.print("  Altitude above ground level ");
  Serial.print(actual_altitude);
  Serial.print("  m ");

  Serial.println();
  
  mySensorData = SD.open("PTData.txt", FILE_WRITE);// open the text file
  mySensorData.print("Temperature = ");// write data to the text file
  mySensorData.print(bmp.temperature);
  mySensorData.print(" *C");

  mySensorData.print("Pressure = ");
  mySensorData.print(bmp.pressure / 100.0);
  mySensorData.print(" hPa");

  mySensorData.print("Approx. Altitude = ");
  mySensorData.print(bmp.readAltitude(SEALEVELPRESSURE_HPA));
  mySensorData.print(" m");

  mySensorData.print("  Altitude above ground level ");
  mySensorData.print(actual_altitude);
  mySensorData.println("  m ");

  mySensorData.close();// close the file
  
  delay(100);// set a delay between readings
}// end the loop
